require("version.nut");
require("town.nut");

/* Import SuperLib for GameScript */
import("util.superlib", "SuperLib", 19);
Result <- SuperLib.Result;
Log <- SuperLib.Log;
Helper <- SuperLib.Helper;
ScoreList <- SuperLib.ScoreList;
Tile <- SuperLib.Tile;
Direction <- SuperLib.Direction;
//Town <- SuperLib.Town; // conflicts with Town class of this GS
Industry <- SuperLib.Industry;

class TestGoal extends GSController 
{
	constructor()
	{
	}

	towns = null;
	last_date = 0;
}

function TestGoal::Start()
{
	local start_tick = GSController.GetTick();

	GSGameSettings.SetValue("economy.town_growth_rate", 2);
	GSGameSettings.SetValue("economy.fund_buildings", 0);
	this.last_date = GSDate.GetCurrentDate();

	// Create the towns list
	Log.Info("Create town list ... (can take a while on large maps)", Log.LVL_INFO);
	this.towns = this.CreateTownList();

	// Set initial time the towns will grow
	foreach (town in this.towns) {
		town.check_timer = GSBase.RandRange(20) + 10;
		town.ManageTown();
	}

	local setup_duration = GSController.GetTick() - start_tick;

	Log.Info("Game setup done", Log.LVL_INFO);
	Log.Info("Setup took " + setup_duration + " ticks", Log.LVL_INFO);
	Log.Info("", Log.LVL_INFO);

	Log.Info("Happy playing", Log.LVL_INFO);

	while (true) {
		this.ManageTowns();
		this.Sleep(60);
	}
}

function TestGoal::CreateTownList()
{
	local result = [];

	/* Make a squirrel array of all towns.
	 * For each town, an instance of Town is created
	 * to store the data related to that town
	 */
	local all_towns = GSTownList();
	foreach(t, _ in all_towns)
	{
		result.append(Town(t));
	}

	return result;
}

function TestGoal::ManageTowns()
{
	local date = GSDate.GetCurrentDate();
	local diff = date - this.last_date;
	this.last_date = date;

	if (diff == 0) return;

	foreach (town in this.towns) {

		town.check_timer -= diff;

		if (town.check_timer <= 0) {
			town.check_timer = town.check_timer + GSBase.RandRange(20) + 10;
			town.ManageTown();
		}
	}
}
